<!DOCTYPE html>
<html lang="en">
<head>
	<meta charset="utf-8">
	<title>ORES</title>
	<meta name="viewport" content="width=device-width, initial-scale=1">
	<meta http-equiv="X-UA-Compatible" content="IE=edge" />
	<base href="/" target="_blank">
	<link rel="stylesheet" href="wikimedia-ui-static/MW/mediawiki.min.css">
	<link rel="import" href="wikimedia-ui-static/MW/elements.html">
<style>
.checkbox{
      display: inline-block;
    }
    .checkbox label{
      padding-right: 20px;
    }
</style>
</head>
<body style="background-color: #f9f9f9">
<div class="col-md-4 col-md-offset-4" style="padding-top: 3em;">

<div style="text-align: center; color: #555;">
<h1 style="margin-bottom: 0;">ORES</h1>
<p style="font-size: 95%; padding-bottom: 2em; letter-spacing: 1px;">
  Objective Revision Evaluation Service
</p>
</div>

	<div class="btn-group" id="wikisList">
		<a class="btn btn-default dropdown-toggle" data-toggle="dropdown" id="wikiDropDownInput" disabled="disabled">Select a wiki<wiki-icon icon="caret-down"></wiki-icon>
		</a>
		<ul class="dropdown-menu" id="wikis">
		</ul>
	</div>

 <div style="padding-top: 10px">
<h5>Insert RevID</h5>
  <div class="form-group">
  <textarea class="form-control" rows="5" placeholder="Insert up to 50 RevIDs separated by a comma" disabled="disabled" id="revIds"></textarea>
</div>
</div>


<button is='wiki-Button' type="submit" button-type="primary" style="float: right;" disabled="disabled" id="resultButton">Give me results!</button><br clear="all" />

</div><br id="afterThis"/>

</body>

<!-- ================================================== -->
<!-- Placed at the end of the document so the pages load faster -->
<script src="static/lib/jquery-3.0.0.min.js"></script>
<script src="static/lib/jquery.tablesorter-2.26.4.min.js"></script>
<script src="wikimedia-ui-static/bootstrap-3.3.5-dist/js/bootstrap.min.js"></script>
<script src="wikimedia-ui-static/bootstrap-3.3.5-dist/js/ie10-viewport-bug-workaround.js"></script>
<script src="/static/js/scorer.js"></script>
