from .about import (__author__, __author_email__, __description__, __name__,
                    __url__, __version__)

__all__ = [__name__, __version__, __author__, __author_email__,
           __description__, __url__]
